@@ -26,6 +26,7 @@
             aa-map-reverse
             aa-map-values
             aa-search
+            aa-search-wrapped
             aa-to-list
             list-to-aa-tree))
 
@@ -178,20 +179,35 @@ trees inside values."
          ((equal? k (key tree)) (replace-value tree v))
          (#t (replace-right-tree tree (aa-insert (right-tree tree) k v))))))))
 
-(define (aa-search tree k)
+(define (aa-search-wrapped tree k)
   "Searches for the node in the specified key, and returns
 the (wrapped) value of that node. If a node is found, the value will
 be returned wrapped in a single element list. If no such node is
 found, a #nil value will be returned.
-If a list of keys is provided instead of a single key, aa-search will
+If a list of keys is provided instead of a single key, aa-search-wrapped will
 attempt a nested tree search, starting the search with the first key
 in the list. If a nested tree of some required level does not exist,
 the function will simply return #nil."
-  (if (list? k) (aa-search-nested tree k)
+  (if (list? k) (aa-search-wrapped-nested tree k)
       (cond
        ((nil? tree) #nil)
-       ((< k (key tree)) (aa-search (left-tree tree) k))
+       ((< k (key tree)) (aa-search-wrapped (left-tree tree) k))
        ((equal? k (key tree)) (list (value tree)))
+       (#t (aa-search-wrapped (right-tree tree) k)))))
+
+(define (aa-search tree k)
+  "Searches for the node in the specified key, and returns the value
+of that node. If no such node is found, a key-not-found exception will
+be thrown, with the key as the first argument.  If a list of keys is
+provided instead of a single key, aa-search-wrapped will attempt a nested tree
+search, starting the search with the first key in the list. If a
+nested tree of some required level does not exist, the function throws
+an exception."
+  (if (list? k) (aa-search-nested tree k)
+      (cond
+       ((nil? tree) (throw 'key-not-found k))
+       ((< k (key tree)) (aa-search (left-tree tree) k))
+       ((equal? k (key tree)) (value tree))
        (#t (aa-search (right-tree tree) k)))))
 
 ;; aa-delete operation
@@ -315,7 +331,7 @@ key from smallest to greatest."
 ;; aa-insert tree key1 key2 value =
 ;; (aa-insert Ao k1 B)
 ;;   where B = (aa-insert Bo k2 value)
-;;         Bo = (aa-search Ao k1)
+;;         Bo = (aa-search-wrapped Ao k1)
 ;;         Ao = tree
 ;;
 ;; That idea is extrapolated out to as many keys as we want using a
@@ -332,7 +348,7 @@ key from smallest to greatest."
     (aa-insert tree
                (car key-lst)
                (aa-insert-nested (let ((search-result
-                                        (aa-search tree (car key-lst))))
+                                        (aa-search-wrapped tree (car key-lst))))
                                    (if (nil? search-result) #nil
                                        (car search-result)))
                                  (cdr key-lst) val)))))
@@ -347,13 +363,27 @@ key from smallest to greatest."
    ;; multiple key list
    (#t
     (let ((search-result
-           (aa-search tree (car key-lst))))
+           (aa-search-wrapped tree (car key-lst))))
       (if (nil? search-result) tree
           (aa-insert tree
                      (car key-lst)
                      (aa-delete-nested (car search-result)
                                        (cdr key-lst))))))))
 
+(define (aa-search-wrapped-nested tree key-lst)
+  (cond
+   ;; no keys, doesn't make sense
+   ((nil? key-lst) (throw 'out-of-range))
+   ;; single key list
+   ((nil? (cdr key-lst))
+    (aa-search-wrapped tree (car key-lst)))
+   ;; multiple key list
+   (#t
+    (let ((search-result
+           (aa-search-wrapped tree (car key-lst))))
+      (if (nil? search-result) #nil
+          (aa-search-wrapped-nested (car search-result) (cdr key-lst)))))))
+
 (define (aa-search-nested tree key-lst)
   (cond
    ;; no keys, doesn't make sense
@@ -362,11 +392,9 @@ key from smallest to greatest."
    ((nil? (cdr key-lst))
     (aa-search tree (car key-lst)))
    ;; multiple key list
-   (#t
-    (let ((search-result
-           (aa-search tree (car key-lst))))
-      (if (nil? search-result) #nil
-          (aa-search-nested (car search-result) (cdr key-lst)))))))
+   (#t (aa-search-nested
+        (aa-search tree (car key-lst))
+        (cdr key-lst)))))
 
 (define (list-to-aa-tree init-tree lst)
 "Takes keys and values from lst and inserts them into
