@@ -32,10 +32,39 @@
                    ("m" . 14)
                    ("e" . update)
                    ("f" . 6)))))
-      (fold (lambda (k l) (cons (aa-search tree k) l)) '()
+      (fold (lambda (k l) (cons (aa-search-wrapped tree k) l)) '()
             (list "m" "h" "f" "e" "b" "q" "t")))
   '(#nil (17) (2) (update) (6) (8) (14)))
 
+(test-equal
+    (let ((tree
+           (fold (lambda (kv t) (aa-insert t (car kv) (cdr kv))) #nil
+                 '(("e" . 5)
+                   ("b" . 2)
+                   ("q" . 17)
+                   ("h" . 8)
+                   ("m" . 14)
+                   ("e" . update)
+                   ("f" . 6)))))
+      (fold (lambda (k l) (cons (aa-search tree k) l)) '()
+            (list "m" "h" "f" "e" "b" "q")))
+  '(17 2 update 6 8 14))
+
+(test-equal
+    (let ((tree
+           (fold (lambda (kv t) (aa-insert t (car kv) (cdr kv))) #nil
+                 '(("e" . 5)
+                   ("b" . 2)
+                   ("q" . 17)
+                   ("h" . 8)
+                   ("m" . 14)
+                   ("e" . update)
+                   ("f" . 6)))))
+      (catch 'key-not-found
+        (lambda () (aa-search tree "t"))
+        (lambda (k e) (list k e))))
+  '(key-not-found "t"))
+
 (test-equal
     (let* ((tree1
             (fold (lambda (kv t) (aa-insert t (car kv) (cdr kv))) #nil
@@ -46,7 +75,7 @@
            (tree2
             (fold (lambda (k t) (aa-delete t k)) tree1
                   '(8 2))))
-      (fold (lambda (k l) (cons (aa-search tree2 k) l)) '()
+      (fold (lambda (k l) (cons (aa-search-wrapped tree2 k) l)) '()
             (list 3 2 4 8)))
   '(#nil ("four") #nil ("three")))
 
@@ -94,7 +123,7 @@
              (aa-insert
               (aa-insert #nil 3 "three") 1 "one") 5 "five"))
            (tree2 (aa-insert tree1 (list 4) "four")))
-      (aa-search tree2 4))
+      (aa-search-wrapped tree2 4))
   (list "four"))
 
 (test-equal
@@ -107,7 +136,7 @@
                               (aa-insert
                                (aa-insert #nil "b" 2) "f" 6) "c" 3)))
            (tree3 (aa-insert tree2 (list 4 "e") 5)))
-      (aa-search (car (aa-search tree3 4)) "e"))
+      (aa-search-wrapped (car (aa-search-wrapped tree3 4)) "e"))
   (list 5))
 
 (test-equal
@@ -125,7 +154,7 @@
                 (aa-insert #nil 'b 2) 'e 5) 'd 4))))
            (tree2
             (aa-insert tree1 (list 5 "b" 'h) 8)))
-      (aa-search (car (aa-search (car (aa-search tree2 5)) "b")) 'h))
+      (aa-search-wrapped (car (aa-search-wrapped (car (aa-search-wrapped tree2 5)) "b")) 'h))
   '(8))
 
 (test-assert
@@ -135,9 +164,9 @@
               (aa-insert #nil 3 "three") 1 "one") 5 "five"))
            (tree2 (aa-delete tree1 (list 1))))
       (and
-       (equal? (aa-search tree2 1) #nil)
-       (equal? (aa-search tree2 5) '("five"))
-       (equal? (aa-search tree2 3) '("three")))))
+       (equal? (aa-search-wrapped tree2 1) #nil)
+       (equal? (aa-search-wrapped tree2 5) '("five"))
+       (equal? (aa-search-wrapped tree2 3) '("three")))))
 
 (test-assert
     (let* ((tree1
@@ -150,9 +179,9 @@
                                (aa-insert #nil "b" 2) "f" 6) "c" 3)))
            (tree3 (aa-delete tree2 (list 4 "f"))))
       (and
-       (equal? (aa-search (car (aa-search tree3 4)) "b") '(2))
-       (equal? (aa-search (car (aa-search tree3 4)) "f") #nil)
-       (equal? (aa-search (car (aa-search tree3 4)) "c") '(3)))))
+       (equal? (aa-search-wrapped (car (aa-search-wrapped tree3 4)) "b") '(2))
+       (equal? (aa-search-wrapped (car (aa-search-wrapped tree3 4)) "f") #nil)
+       (equal? (aa-search-wrapped (car (aa-search-wrapped tree3 4)) "c") '(3)))))
 
 (test-assert
     (let* ((tree1
@@ -171,11 +200,27 @@
             (aa-delete tree1 (list 5 "b" 'b))))
       (and
        (equal?
-        (aa-search (car (aa-search (car (aa-search tree2 5)) "b")) 'd) '(4))
+        (aa-search (aa-search (aa-search tree2 5) "b") 'd) 4)
        (equal?
-        (aa-search (car (aa-search (car (aa-search tree2 5)) "b")) 'b) #nil)
+        (aa-search-wrapped (car (aa-search-wrapped (car (aa-search-wrapped tree2 5)) "b")) 'd) '(4))
        (equal?
-        (aa-search (car (aa-search (car (aa-search tree2 5)) "b")) 'e) '(5)))))
+        (aa-search tree2 '(5 "b" d)) 4)
+       (equal?
+        (aa-search-wrapped tree2 '(5 "b" d)) '(4))
+       (equal?
+        (aa-search-wrapped (car (aa-search-wrapped (car (aa-search-wrapped tree2 5)) "b")) 'b) #nil)
+       (equal?
+        (catch 'key-not-found
+          (lambda () (aa-search (aa-search (aa-search tree2 5) "b") 'b))
+          (lambda (k a) (list k a)))
+        '(key-not-found b))
+       (equal?
+        (catch 'key-not-found
+          (lambda () (aa-search tree2 '(5 "b" b)))
+          (lambda (k a) (list k a)))
+        '(key-not-found b))
+       (equal?
+        (aa-search-wrapped (car (aa-search-wrapped (car (aa-search-wrapped tree2 5)) "b")) 'e) '(5)))))
 
 (test-equal
     (let* ((tree1
@@ -190,7 +235,7 @@
               (aa-insert
                (aa-insert
                 (aa-insert #nil 'b 2) 'e 5) 'd 4)))))
-      (aa-search tree1 (list 5 "b" 'e)))
+      (aa-search-wrapped tree1 (list 5 "b" 'e)))
   '(5))
 
 (test-equal
@@ -206,7 +251,7 @@
               (aa-insert
                (aa-insert
                 (aa-insert #nil 'b 2) 'e 5) 'd 4)))))
-      (aa-search tree1 (list 5 #:b 'e)))
+      (aa-search-wrapped tree1 (list 5 #:b 'e)))
   '(5))
 
 (test-equal
@@ -222,7 +267,7 @@
               (aa-insert
                (aa-insert
                 (aa-insert #nil 'b 2) 'e 5) 'd 4)))))
-      (aa-search tree1 (list 5 "q" 'e)))
+      (aa-search-wrapped tree1 (list 5 "q" 'e)))
   #nil)
 
 (define-class <coin> ()
@@ -247,8 +292,8 @@
            (aa-insert
             (aa-insert
              (aa-insert #nil c1 "head1") c2 "tail1") c3 "head2")))
-      (and (equal? (aa-search tree1 c1) '("head2"))
-           (equal? (aa-search tree1 c2) '("tail1"))))))
+      (and (equal? (aa-search-wrapped tree1 c1) '("head2"))
+           (equal? (aa-search-wrapped tree1 c2) '("tail1"))))))
 
 (test-equal
     (let ((tree1
@@ -257,7 +302,7 @@
                   #:f
                   ("d" 5
                    "b" (_ foo bar))))))
-      (aa-search tree1 '(#:f "b")))
+      (aa-search-wrapped tree1 '(#:f "b")))
   '((foo bar)))
 
 (test-end "test-aa-tree")
